//A basic "immediate mode" canvas for blitting.
//Has support for blitting subsections of BitmapData objects, as well as rudimentary text / font output.

package  
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.DisplayObjectContainer;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextLineMetrics;
	
	/**
	 * ...
	 * @author ...
	 */
	public class Canvas 
	{
		public function Canvas(aWidth:int, aHeight:int, aParent:DisplayObjectContainer)
		{
			init(aWidth, aHeight, aParent);
		}
		
		public function lock():void
		{
			myCanvas.lock();
		}
		
		public function unlock():void
		{
			myCanvas.unlock();
		}
		
		public function setPosition(aX:Number, aY:Number):void
		{
			myCanvasBitmap.x = aX;
			myCanvasBitmap.y = aY;
		}
		
		public function clear(aX:int, aY:int, aWidth:int, aHeight:int, aColor:uint):void
		{
			mySrc.x = aX;
			mySrc.y = aY;
			mySrc.width = aWidth;
			mySrc.height = aHeight;
			
			myCanvas.fillRect(mySrc, aColor);
		}
		
		public function setPixel(aX:int, aY:int, aColor:uint):void
		{
			myCanvas.setPixel(aX, aY, aColor);
		}
		
		public function blit(aBitmap:BitmapData,
							aDstX:int, aDstY:int,
							aCopyWidth:int = 0, aCopyHeight:int = 0,
							aSrcX:int = 0, aSrcY:int = 0):void
		{
			mySrc.x = aSrcX;
			mySrc.y = aSrcY;
			
			mySrc.width = aCopyWidth ? aCopyWidth : aBitmap.width;
			mySrc.height = aCopyHeight ? aCopyHeight : aBitmap.height;
			
			myDst.x = aDstX;
			myDst.y = aDstY;
			
			myCanvas.copyPixels(aBitmap, mySrc, myDst);
		}
		
		public function text(aDstX:int, aDstY:int, aText:String, aColor:uint):void
		{
			myText.text = aText;

			const metrics:TextLineMetrics = myText.getLineMetrics(0);
			var m:Matrix = new Matrix();
			
			m.translate(aDstX, aDstY);
			myText.textColor = 0xff000000;
			myCanvas.draw(myText, m);
			
			m.translate( -1, -1);
			myText.textColor = aColor;
			myCanvas.draw(myText, m);
		}
		
		public function width():int
		{
			return myCanvas.width;
		}
		
		public function height():int
		{
			return myCanvas.height;
		}
		
		//private
		private function init(aWidth:int, aHeight:int, aParent:DisplayObjectContainer):void
		{
			//canvas (24 bit, i.e. not transparent)
			myCanvas = new BitmapData(aWidth, aHeight, false, 0x000000);
			myCanvasBitmap = new Bitmap(myCanvas);
			aParent.addChild(myCanvasBitmap);
			
			//text
			var format:TextFormat = new TextFormat;
			format.font = "arial";
			format.size = TEXT_HEIGHT;
			format.bold = true;
			
			myText = new TextField;
			myText.defaultTextFormat = format;
			myText.width = aWidth;
			myText.height = 32;
			myText.background = false;
		}
		
		//constants
		public static const TEXT_HEIGHT:int = 24;
		
		//members
		private var mySrc:Rectangle = new Rectangle;
		private var myDst:Point = new Point;
		private var myCanvas:BitmapData = null;
		private var myCanvasBitmap:Bitmap = null;
		private var myText:TextField = null;
	}
}