//This class registers callbacks to various input related events and caches the results.
//The intent is to poll keyDown() and keyPressed().
//update() needs to be called once per tick / frame for the keyPressed() stuff to work.

package  
{
	import flash.events.EventDispatcher;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	
	/**
	 * ...
	 * @author johno
	 */
	public class Input 
	{
		public static function registerCallbacks(aDispatcher:EventDispatcher):void
		{
			aDispatcher.addEventListener(MouseEvent.MOUSE_DOWN, cbLMBDown);
			aDispatcher.addEventListener(MouseEvent.MOUSE_UP, cbLMBUp);
			aDispatcher.addEventListener(MouseEvent.MOUSE_MOVE, cbMouseMove);
			aDispatcher.addEventListener(KeyboardEvent.KEY_DOWN, cbKeyDown);
			aDispatcher.addEventListener(KeyboardEvent.KEY_UP, cbKeyUp);
		}
		
		private static function cbKeyDown(anEvent:KeyboardEvent):void
		{
			myKeyDown[anEvent.keyCode] = true;
		}
		
		private static function cbKeyUp(anEvent:KeyboardEvent):void
		{
			myKeyDown[anEvent.keyCode] = false;
		}
		
		private static function cbLMBDown(anEvent:MouseEvent):void
		{
			myLMBDown = true;
		}
		
		private static function cbLMBUp(anEvent:MouseEvent):void
		{
			myLMBDown = false;
		}
		
		private static function cbMouseMove(anEvent:MouseEvent):void
		{
			myCursorX = anEvent.stageX;
			myCursorY = anEvent.stageY;
		}
		
		public static function update():void
		{
			myLMBDownLast = myLMBDown;
			//myLMBDown = false;
			
			var i:uint;
			for (i = 0; i < 256; i++)
			{
				myKeyDownLast[i] = myKeyDown[i];
				//myKeyDown[i] = false;
			}
		}
		
		public static function keyDown(aKeyCode:uint):Boolean
		{
			return myKeyDown[aKeyCode];
		}
		
		public static function keyPressed(aKeyCode:uint):Boolean
		{
			return myKeyDown[aKeyCode] && !myKeyDownLast[aKeyCode];
		}
		
		//constants
		//I got these from: http://www.dakmm.com/?p=272
		public static const KEY_ESCAPE:uint = 27,
							KEY_ENTER:uint = 13,
							KEY_PAGEUP:uint = 33,
							KEY_PAGEDOWN:uint = 34,
							KEY_END:uint = 35,
							KEY_HOME:uint = 36,
							KEY_F1:uint = 112,
							KEY_F2:uint = 113,
							KEY_F3:uint = 114,
							KEY_F4:uint = 115,
							KEY_F5:uint = 116,
							KEY_F6:uint = 117,
							KEY_F7:uint = 118,
							KEY_F8:uint = 119,
							KEY_F9:uint = 120,
							KEY_F11:uint = 122,
							KEY_F12:uint = 123,
							KEY_1:uint = 49,
							KEY_2:uint = 50,
							KEY_3:uint = 51,
							KEY_4:uint = 52,
							KEY_A:uint = 65,
							KEY_D:uint = 68,
							KEY_H:uint = 72,
							KEY_S:uint = 83,
							KEY_T:uint = 84,
							KEY_W:uint = 87,
							KEY_SHIFT:uint = 16,
							KEY_CONTROL:uint = 17,
							KEY_UP:uint = 38,
							KEY_DOWN:uint = 40,
							KEY_LEFT:uint = 37,
							KEY_RIGHT:uint = 39,
							KEY_SPACE:uint = 32;
		
		//members
		public static var myLMBDown:Boolean = false,
						myLMBDownLast:Boolean = false,
						myCursorX:int = 0,
						myCursorY:int = 0,
						myKeyDown:Vector.<Boolean> = new Vector.<Boolean>(256),
						myKeyDownLast:Vector.<Boolean> = new Vector.<Boolean>(256);
	}
}