package 
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	
	/**
	 * ...
	 * @author johno
	 */
	public class Main extends Sprite 
	{
		public function Main():void 
		{
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		//setup
		private function init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			// entry point
			Input.registerCallbacks(stage);
			myHero = new Hero(stage.stageWidth * 0.5, stage.stageWidth * 0.5);
			myTiles = (new TILES as Bitmap).bitmapData;		//this is how to load embedded assets...
			myThings = (new THINGS as Bitmap).bitmapData;	//this is how to load embedded assets...
			myCanvas = new Canvas(stage.stageWidth, stage.stageHeight, this)
			addEventListener(Event.ENTER_FRAME, tick);
		}
		
		//per frame / tick callback
		private function tick(e:Event = null):void
		{	
			Input.update();
			myHero.update(1 / stage.frameRate, Input.keyDown(Input.KEY_A), Input.keyDown(Input.KEY_D), Input.keyDown(Input.KEY_W), Input.keyDown(Input.KEY_S));
			draw();
		}
		
		private function draw():void
		{
			//we want the hero in the center of the canvas, world units are the same as screen units (pixels)...
			const TARGET_VIEW_X:Number = myHero.myX - stage.stageWidth * 0.5;
			const TARGET_VIEW_Y:Number = myHero.myY - stage.stageHeight * 0.5;
			
			//we interpolate the view...
			const TIME_PER_TICK:Number = 1 / stage.frameRate;
			myViewX += (TARGET_VIEW_X - myViewX) * VIEW_LERP_SPEED * TIME_PER_TICK;
			myViewY += (TARGET_VIEW_Y - myViewY) * VIEW_LERP_SPEED * TIME_PER_TICK;
			
			//apparently locking is good for performance...
			myCanvas.lock();
			
				drawMap(myViewX, myViewY);
				drawThings(myViewX, myViewY);
				drawHero(myViewX, myViewY);
				
				var y:int;
				myCanvas.text(0, y = 0, "Blitting in AS3", 0xffffff);
				
				if (0)
				{
					myCanvas.text(0, y = myCanvas.height() - Canvas.TEXT_HEIGHT * 4, "hero x " + (int)(myHero.myX), 0xffffff);
					myCanvas.text(0, y += Canvas.TEXT_HEIGHT, "hero y " + (int)(myHero.myY), 0xffffff);
					myCanvas.text(0, y += Canvas.TEXT_HEIGHT, "view x " + (int)(myViewX), 0xffffff);
					myCanvas.text(0, y += Canvas.TEXT_HEIGHT, "view y " + (int)(myViewY), 0xffffff);
				}
				else
				{
					myCanvas.text(0, y = myCanvas.height() - Canvas.TEXT_HEIGHT - 6, "johno@johno.se", 0xffffff);
				}
				
			myCanvas.unlock();
		}
		
		//this is based on: http://www.johno.se/software/16cases.php
		private function drawMap(aViewX:Number, aViewY:Number):void
		{
			var x:int, y:int;
			var flags:uint;
			
			//we want to draw enough tiles to cover the screen, clipping is handled automatically...
			const START_X:int = aViewX / TILE_ASPECT - 1;
			const START_Y:int = aViewY / TILE_ASPECT - 1;
			for (y = START_Y; y < START_Y + 22; y++)
			{
				for (x = START_X; x < START_X + 22; x++)
				{
					flags = 0;
					
					if (stone(x, y))
					{
						flags += 1;	//top left corner == 1
					}
					if (stone(x + 1, y))
					{
						flags += 2;	//top right corner == 2
					}
					if (stone(x, y + 1))
					{
						flags += 4;	//bottom left corner == 4
					}
					if (stone(x + 1, y + 1))
					{
						flags += 8;	//bottom right corner == 8
					}
					
					//all permutation of the above is 16 cases...
					switch(flags)
					{
					case 15:	drawTileShift(aViewX, aViewY, x, y, 1, 1);	break;
					case 03:	drawTileShift(aViewX, aViewY, x, y, 1, 2);	break;
					case 12:	drawTileShift(aViewX, aViewY, x, y, 1, 0);	break;
					case 10:	drawTileShift(aViewX, aViewY, x, y, 0, 1);	break;
					case 05:	drawTileShift(aViewX, aViewY, x, y, 2, 1);	break;
					case 08:	drawTileShift(aViewX, aViewY, x, y, 0, 0);	break;
					case 04:	drawTileShift(aViewX, aViewY, x, y, 2, 0);	break;
					case 02:	drawTileShift(aViewX, aViewY, x, y, 0, 2);	break;
					case 01:	drawTileShift(aViewX, aViewY, x, y, 2, 2);	break;
					case 00:	drawTileShift(aViewX, aViewY, x, y, 3, 0);	break;
					case 06:	drawTileShift(aViewX, aViewY, x, y, 3, 1);	break;
					case 09:	drawTileShift(aViewX, aViewY, x, y, 3, 2);	break;
					case 14:	drawTileShift(aViewX, aViewY, x, y, 0, 3);	break;
					case 13:	drawTileShift(aViewX, aViewY, x, y, 1, 3);	break;
					case 11:	drawTileShift(aViewX, aViewY, x, y, 2, 3);	break;
					case 07:	drawTileShift(aViewX, aViewY, x, y, 3, 3);	break;
					}
				}
			}			
		}
		
		private function drawThings(aViewX:Number, aViewY:Number):void
		{
			var x:int, y:int;
			var map:Number;
			
			//we want to draw enough tiles to cover the screen, clipping is handled automatically...
			const START_X:int = aViewX / TILE_ASPECT - 1;
			const START_Y:int = aViewY / TILE_ASPECT - 2;	//things are taller than TILE_ASPECT...
			for (y = START_Y; y < START_Y + 23; y++)		//things are taller than TILE_ASPECT...
			{
				for (x = START_X; x < START_X + 22; x++)
				{
					map = fractalNoise(x, y);
					if (map > THING_THRESHOLD)
					{
						//observe the standard "world to screen space" transform is: screen coord = world coord - view coord (in both x and y)...
						myCanvas.blit(myThings, x * TILE_ASPECT - aViewX, y * TILE_ASPECT - aViewY, TILE_ASPECT, 0, TILE_ASPECT * ((int)(Math.abs(map * 10)) % 4));
					}
				}
			}			
		}
		
		private function drawHero(aViewX:Number, aViewY:Number):void
		{
			//observe the standard "world to screen space" transform is: screen coord = world coord - view coord (in both x and y)...
			myCanvas.blit(myThings, myHero.myX - aViewX, myHero.myY - aViewY, TILE_ASPECT, 0, TILE_ASPECT * 4, 0);
		}
		
		private function stone(aX:int, aY:int):Boolean
		{
			return fractalNoise(aX, aY) > STONE_THRESHOLD;
		}

		//this generates the pseudo-random terrain using Simplex noise (http://en.wikipedia.org/wiki/Simplex_noise)
		//you can play around with the number of octaves (calls to noise3()) as well as the frequency scale of the x and y coordinates...
		//the original noise3() function returns a value between -1 and +1, so observe that summing several values may exceed that range...
		private function fractalNoise(aX:int, aY:int):Number
		{
			return PerlinSimplex.ourInstance.noise3(aX * 0.05, aY * 0.05, 0) +
				PerlinSimplex.ourInstance.noise3(aX * 0.1, aY * 0.1, 0) +
				PerlinSimplex.ourInstance.noise3(aX * 1., aY * 1., 0);
		}
		
		private function drawTileShift(aViewX:Number, aViewY:Number, aDstX:int, aDstY:int, aSrcX:int, aSrcY:int):void
		{
			//observe the standard "world to screen space" transform is: screen coord = world coord - view coord (in both x and y)...
			myCanvas.blit(myTiles, aDstX * TILE_ASPECT - aViewX, aDstY * TILE_ASPECT - aViewY, TILE_ASPECT, TILE_ASPECT, aSrcX * TILE_ASPECT, aSrcY * TILE_ASPECT);
		}
		
		//assets
		//graphics from: http://www.lostgarden.com/search/label/free%20game%20graphics
		[Embed(source = "16cases_pretty.png")] private static const TILES:Class;	//embed assets in the .swf...
		[Embed(source = "things.png")] private static const THINGS:Class;			//embed assets in the .swf...
		
		//constants
		private static const TILE_ASPECT:int = 40;
		private static const STONE_THRESHOLD:Number = 0.;
		private static const THING_THRESHOLD:Number = 0.7;
		private static const VIEW_LERP_SPEED:Number = 2;
		
		//members
		private var myHero:Hero = null;
		private var myTiles:BitmapData = null;
		private var myThings:BitmapData = null;
		private var myCanvas:Canvas = null;
		private var myViewX:Number = 0;
		private var myViewY:Number = 0;
	}
}